"use strict";

/* eslint-disable no-console */

import React, { memo, useCallback, useEffect, useState } from 'react';
import { ScrollView, StyleSheet, Text, View } from 'react-native';
import { useShiki } from './SyntaxContext';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const DEBUG = true;
function _log(...args) {
  if (DEBUG) console.log('[SyntaxHighlighter]', ...args);
}
const styles = StyleSheet.create({
  container: {
    flex: 1
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap'
  }
});
export const SyntaxHighlighter = /*#__PURE__*/memo(({
  code,
  language,
  theme = 'github-dark',
  fontSize = 14
}) => {
  const {
    highlighter,
    isReady
  } = useShiki();
  const [tokens, setTokens] = useState([]);
  const [error, setError] = useState(null);
  const highlight = useCallback(async () => {
    if (!highlighter || !isReady) return;
    try {
      // Get loaded languages from highlighter
      const loadedLangs = highlighter.getLoadedLanguages();
      _log('Available languages:', loadedLangs);

      // Normalize language name
      const normalizedLang = language?.toLowerCase() || 'text';
      _log('Requested language:', normalizedLang);

      // Use text if language isn't loaded
      const langToUse = loadedLangs.includes(normalizedLang) ? normalizedLang : 'text';
      _log('Using language:', langToUse);
      const results = await highlighter.codeToTokens(code.trim(), {
        lang: langToUse,
        theme
      });
      const processedTokens = results.tokens.map(line => {
        return line.map(token => ({
          ...token,
          color: token.color,
          fontStyle: token.fontStyle ?? 0,
          explanation: token.explanation ?? []
        }));
      });
      setTokens(processedTokens);
      setError(null);
    } catch (err) {
      console.error('Highlighting error:', err);
      setError(err instanceof Error ? err.message : 'Failed to highlight code');
    }
  }, [code, language, theme, highlighter, isReady]);
  useEffect(() => {
    highlight();
  }, [highlight]);
  useEffect(() => {
    if (tokens.length > 0) {
      _log('Token analysis:', {
        lineCount: tokens.length,
        tokenTypes: tokens.flat().map(t => t.explanation?.[0]?.scopes).flat(),
        colors: [...new Set(tokens.flat().map(t => t.color))]
      });
    }
  }, [tokens]);
  if (!isReady) {
    return /*#__PURE__*/_jsx(View, {
      style: styles.container,
      children: /*#__PURE__*/_jsx(Text, {
        style: {
          fontSize
        },
        children: code
      })
    });
  }
  if (error) {
    return /*#__PURE__*/_jsxs(View, {
      style: styles.container,
      children: [/*#__PURE__*/_jsx(Text, {
        style: {
          color: 'red'
        },
        children: error
      }), /*#__PURE__*/_jsx(Text, {
        style: {
          fontSize
        },
        children: code
      })]
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    horizontal: true,
    style: styles.container,
    children: /*#__PURE__*/_jsx(View, {
      children: tokens.map((line, i) => /*#__PURE__*/_jsx(View, {
        style: styles.line,
        children: line.map((token, j) => /*#__PURE__*/_jsx(Text, {
          style: {
            color: token.color,
            fontSize,
            ...(token.fontStyle && token.fontStyle & 1 && {
              fontStyle: 'italic'
            }),
            ...(token.fontStyle && token.fontStyle & 2 && {
              fontWeight: 'bold'
            }),
            ...(token.bgColor && {
              backgroundColor: token.bgColor
            })
          },
          children: token.content
        }, `${i}-${j}`))
      }, i))
    })
  });
});
//# sourceMappingURL=SyntaxHighlighter.js.map