import type { HighlighterCore, LanguageRegistration, ThemeRegistrationAny } from '@shikijs/types'
import type React from 'react'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import { createContext, useContext, useEffect, useMemo, useState } from 'react'

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
}

interface ShikiProviderProps {
  children: React.ReactNode
  initialLanguages: LanguageRegistration[]
  initialThemes: ThemeRegistrationAny[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

export function ShikiProvider({ children, initialLanguages, initialThemes }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    const initHighlighter = async () => {
      try {
        // Process themes
        const processedThemes = initialThemes.map(theme => ({
          ...theme,
          settings: theme.settings || [],
          tokenColors: theme.tokenColors || [],
        }))

        // Create highlighter with correct typing
        const highlighterInstance = await createHighlighterCore({
          langs: initialLanguages,
          themes: processedThemes,
          engine: createJavaScriptRegexEngine({
            forgiving: true,
            cache: new Map(),
          }),
        })

        // Pre-load languages and themes with proper type handling
        await Promise.all([...initialLanguages.map(lang => highlighterInstance.loadLanguage(lang)), ...processedThemes.map(theme => highlighterInstance.loadTheme(theme))])

        setHighlighter(highlighterInstance)
        setIsReady(true)
        setError(null)
      }
      catch (err) {
        console.error('Highlighter initialization failed:', err)
        setError(err instanceof Error ? err : new Error('Initialization failed'))
        setIsReady(false)
      }
    }

    initHighlighter()
  }, [initialLanguages, initialThemes])

  const value = useMemo<ShikiContextValue>(
    () => ({
      highlighter,
      isReady,
      error,
    }),
    [highlighter, isReady, error],
  )

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}
