/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import React, { memo, useCallback, useEffect, useState } from 'react'
import { ScrollView, StyleSheet, Text, View } from 'react-native'
import { useShiki } from './SyntaxContext'

const DEBUG = true
function _log(...args: any[]) {
  if (DEBUG)
    console.log('[SyntaxHighlighter]', ...args)
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
})

interface SyntaxHighlighterProps {
  code: string
  language: string
  theme?: string | ThemeRegistrationAny
  fontSize?: number
}

export const SyntaxHighlighter = memo(({ code, language, theme = 'github-dark', fontSize = 14 }: SyntaxHighlighterProps) => {
  const { highlighter, isReady } = useShiki()
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string | null>(null)

  const highlight = useCallback(async () => {
    if (!highlighter || !isReady)
      return

    try {
      // Get loaded languages from highlighter
      const loadedLangs = highlighter.getLoadedLanguages()
      _log('Available languages:', loadedLangs)

      // Normalize language name
      const normalizedLang = language?.toLowerCase() || 'text'
      _log('Requested language:', normalizedLang)

      // Use text if language isn't loaded
      const langToUse = loadedLangs.includes(normalizedLang) ? normalizedLang : 'text'
      _log('Using language:', langToUse)

      const results = await highlighter.codeToTokens(code.trim(), {
        lang: langToUse,
        theme,
      })

      const processedTokens = results.tokens.map((line) => {
        return line.map(token => ({
          ...token,
          color: token.color,
          fontStyle: token.fontStyle ?? 0,
          explanation: token.explanation ?? [],
        }))
      })

      setTokens(processedTokens)
      setError(null)
    }
    catch (err) {
      console.error('Highlighting error:', err)
      setError(err instanceof Error ? err.message : 'Failed to highlight code')
    }
  }, [code, language, theme, highlighter, isReady])

  useEffect(() => {
    highlight()
  }, [highlight])

  useEffect(() => {
    if (tokens.length > 0) {
      _log('Token analysis:', {
        lineCount: tokens.length,
        tokenTypes: tokens
          .flat()
          .map(t => t.explanation?.[0]?.scopes)
          .flat(),
        colors: [...new Set(tokens.flat().map(t => t.color))],
      })
    }
  }, [tokens])

  if (!isReady) {
    return (
      <View style={styles.container}>
        <Text style={{ fontSize }}>{code}</Text>
      </View>
    )
  }

  if (error) {
    return (
      <View style={styles.container}>
        <Text style={{ color: 'red' }}>{error}</Text>
        <Text style={{ fontSize }}>{code}</Text>
      </View>
    )
  }

  return (
    <ScrollView horizontal style={styles.container}>
      <View>
        {tokens.map((line, i) => (
          <View key={i} style={styles.line}>
            {line.map((token, j) => (
              <Text
                key={`${i}-${j}`}
                style={{
                  color: token.color,
                  fontSize,
                  ...(token.fontStyle && token.fontStyle & 1 && { fontStyle: 'italic' }),
                  ...(token.fontStyle && token.fontStyle & 2 && { fontWeight: 'bold' }),
                  ...(token.bgColor && { backgroundColor: token.bgColor }),
                }}
              >
                {token.content}
              </Text>
            ))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
})
